Package: iTensor
Type: Package
Title: ICA-Based Matrix/Tensor Decomposition
Version: 1.0.5
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                      email = "k.t.the-answer@hotmail.co.jp"))
Description: Some functions for performing ICA, MICA, Group ICA, and Multilinear ICA are implemented.
    ICA, MICA/Group ICA, and Multilinear ICA extract statistically independent components from single matrix, multiple matrices, and single tensor, respectively.
    For the details of these methods, see the reference section of GitHub README.md <https://github.com/rikenbit/iTensor>.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.4.0)
URL: https://github.com/rikenbit/iTensor
RoxygenNote: 7.2.3
Suggests: nnTensor, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Imports: MASS, methods, graphics, utils, stats, rTensor, jointDiag,
        mgcv, einsum, geigen, mixOmics
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-11 08:21:22 UTC; koki
Author: Koki Tsuyuzaki [aut, cre]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Repository: CRAN
Date/Publication: 2026-05-11 16:50:02 UTC
Built: R 4.6.0; ; 2026-05-11 19:33:47 UTC; unix
