# Generated by roxygen2: do not edit by hand

export(CorrIndex)
export(GroupICA)
export(ICA)
export(ICA2)
export(MICA)
export(MultilinearICA)
export(toyModel)
import(mixOmics)
importFrom(MASS,ginv)
importFrom(einsum,einsum)
importFrom(geigen,geigen)
importFrom(graphics,hist)
importFrom(jointDiag,uwedge)
importFrom(methods,is)
importFrom(mgcv,gam)
importFrom(rTensor,as.tensor)
importFrom(rTensor,cs_unfold)
importFrom(rTensor,ttl)
importFrom(rTensor,ttm)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dnorm)
importFrom(stats,filter)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,data)
importFrom(utils,str)
